@@ -1,258 +1,99 @@
-package SVN::Look;
-
+use 5.8.0;
 use strict;
 use warnings;
+
+package SVN::Look;
+{
+  $SVN::Look::VERSION = '0.34';
+}
+# ABSTRACT: A caching wrapper around the svnlook command.
+
 use Carp;
 use File::Spec::Functions;
 
-=head1 NAME
-
-SVN::Look - A caching wrapper aroung the svnlook command.
-
-=head1 VERSION
-
-Version 0.22
-
-=cut
-
-our $VERSION = '0.22';
-
-=head1 SYNOPSIS
-
-  use SVN::Look;
-  my $revlook = SVN::Look->new('/repo/path', -r => 123);
-  my $author  = $revlook->author();
-  my $msg     = $revlook->log_msg();
-  my @added_files   = $revlook->added();
-  my @updated_files = $revlook->updated();
-  my @deleted_files = $revlook->deleted();
-  my @changed_files = $revlook->changed();
-  my $file_contents = $revlook->cat('/path/to/file/in/repository');
-
-  my $trxlook = SVN::Look->new('/repo/path', -t => 123);
-
-=head1 DESCRIPTION
-
-The svnlook command is the workhorse of Subversion hook scripts, being
-used to gather all sorts of information about a repository, its
-revisions, and its transactions. This script provides a simple object
-oriented interface to a specific svnlook invocation, to make it easier
-to hook writers to get and use the information they need. Moreover,
-all the information gathered buy calling the svnlook command is cached
-in the object, avoiding repetitious calls.
-
-=cut
 
 BEGIN {
-    if ($^O eq 'MSWin32') {
-	$ENV{PATH} .= ';C:\Program Files (x86)\VisualSVN Server\bin';
+    my $path = $ENV{PATH} || '';
+    open my $svnlook, 'svnlook --version |' ## no critic (InputOutput::ProhibitTwoArgOpen)
+	or die "Aborting because I couldn't find the 'svnlook' executable in PATH='$path'.\n";
+    $_ = <$svnlook>;
+    if (my ($major, $minor, $patch) = (/(\d+)\.(\d+)\.(\d+)/)) {
+	$major > 1 || $major == 1 && $minor >= 4
+	    or die "I need at least version 1.4.0 of svnlook but you have only $major.$minor.$patch in PATH='$path'.\n";
     } else {
-	$ENV{PATH} .= ':/usr/local/bin:/usr/bin:/bin';
+	die "Can't grok Subversion version from svnlook --version command.\n";
     }
-    eval {
-	open my $pipe, '-|', "svnlook --version" or die;
-	local $/ = undef;		# slurp mode
-	<$pipe>;
-	close $pipe or die;
-    };
-    die "Aborting because I couldn't execute the svnlook command: $@\n"
-	if $@;
+    local $/ = undef;		# slurp mode
+    <$svnlook>;
+    close $svnlook or die "Can't close svnlook commnand.\n";
 }
 
-=head1 METHODS
-
-=over 4
-
-=item B<new> REPO, WHAT, NUMBER
-
-The SVN::Look constructor needs three arguments:
-
-=over
-
-=item REPO is the path to the repository.
-
-=item WHAT must be either '-r' or '-t', specifying if the third
-argument is a revision number or a transaction number, respectivelly.
-
-=item NUMBER is either a revision or transaction NUMBER, as specified
-by WHAT.
-
-=back
-
-=cut
 
 sub new {
-    my ($class, $repo, $what, $txn_or_rev) = @_;
+    my ($class, $repo, @opts) = @_;
     my $self = {
-        repo     => $repo,
-        what     => [$what, $txn_or_rev],
-        txn      => undef,
-        rev      => undef,
-        author   => undef,
-        log      => undef,
-        changed  => undef,
-        proplist => undef,
+        repo => $repo,
+        opts => [@opts],
     };
-    if ($what eq '-t') {
-        $self->{txn} = $txn_or_rev;
-    }
-    elsif ($what eq '-r') {
-        $self->{rev} = $txn_or_rev;
-    }
-    else {
-        croak "Look::new: third argument must be -t or -r, not ($what)";
-    }
     bless $self, $class;
     return $self;
 }
 
 sub _svnlook {
     my ($self, $cmd, @args) = @_;
-    my @cmd = (svnlook => $cmd, $self->{repo});
-    push @cmd, @{$self->{what}} unless $cmd =~ /^(?:youngest|uuid|lock)$/;
-    open my $fd, '-|', '"' . join('"  "', @cmd, @args) . '"'
+    my @cmd = ('svnlook', $cmd, $self->{repo});
+    push @cmd, @{$self->{opts}} unless $cmd =~ /^(?:youngest|uuid|lock)$/;
+    my $args = @args ? '"' . join('" "', @args) . '"' : '';
+    open my $fd, join(' ', @cmd, $args, '|') ## no critic (InputOutput::ProhibitTwoArgOpen)
         or die "Can't exec svnlook $cmd: $!\n";
     if (wantarray) {
         my @lines = <$fd>;
         close $fd or die "Failed closing svnlook $cmd: $!\n";
-        chomp foreach @lines;
+        chomp @lines;
         return @lines;
     }
     else {
         local $/ = undef;
         my $line = <$fd>;
         close $fd or die "Failed closing svnlook $cmd: $!\n";
-        chomp $line;
+        chomp $line unless $cmd eq 'cat';
         return $line;
     }
 }
 
-=item B<repo>
-
-Returns the repository path that was passed to the constructor.
-
-=cut
 
 sub repo {
     my $self = shift;
     return $self->{repo};
 }
 
-=item B<txn>
-
-Returns the transaction number that was passed to the constructor. If
-none was passed, returns undef.
-
-=cut
 
 sub txn {
     my $self = shift;
-    return $self->{txn};
+    return $self->{opts}[0] eq '-t' ? $self->{opts}[1] : undef;
 }
 
-=item B<rev>
-
-Returns the revision number that was passed to the constructor. If
-none was passed, returns undef.
-
-=cut
 
 sub rev {
     my $self = shift;
-    return $self->{rev};
+    return $self->{opts}[0] eq '-r' ? $self->{opts}[1] : undef;
 }
 
-=item B<author>
-
-Returns the author of the revision/transaction.
-
-=cut
 
 sub author {
     my $self = shift;
-    unless ($self->{author}) {
+    unless (exists $self->{author}) {
         chomp($self->{author} = $self->_svnlook('author'));
     }
     return $self->{author};
 }
 
-=item B<log_msg>
 
-Returns the log message of the revision/transaction.
-
-=cut
-
-sub log_msg {
-    my $self = shift;
-    unless ($self->{log}) {
-        $self->{log} = $self->_svnlook('log');
-    }
-    return $self->{log};
-}
-
-=item B<date>
-
-Returns the date of the revision/transaction.
-
-=cut
-
-sub date {
-    my $self = shift;
-    unless ($self->{date}) {
-        $self->{date} = ($self->_svnlook('info'))[1];
-    }
-    return $self->{date};
-}
-
-=item B<proplist> PATH
-
-Returns a reference to a hash containing the properties associated with PATH.
-
-=cut
-
-sub proplist {
+sub cat {
     my ($self, $path) = @_;
-    unless ($self->{proplist}{$path}) {
-        my $text = $self->_svnlook('proplist', '--verbose', $path);
-        my @list = split /^\s\s(\S+)\s:\s/m, $text;
-        shift @list;            # skip the leading empty field
-        chomp(my %hash = @list);
-        $self->{proplist}{$path} = \%hash;
-    }
-    return $self->{proplist}{$path};
+    return $self->_svnlook('cat', $path);
 }
 
-=item B<changed_hash>
-
-Returns a reference to a hash containing information about all file
-changes occurred in the revision. The hash always has the following
-keys:
-
-=over
-
-=item added
-
-A list of files added in the revision.
-
-=item deleted
-
-A list of files deleted in the revision.
-
-=item updated
-
-A list of files updated in the revision.
-
-=item prop_modified
-
-A list of files that had properties modified in the revision.
-
-=item copied
-
-A hash containing information about each file or diretory copied in the revision. The hash keys are the names of elements copied to. The value associated with a key is a two-element array containing the name of the element copied from and the specific revision from which it was copied.
-
-=back
-
-=cut
 
 sub changed_hash {
     my $self = shift;
@@ -291,57 +132,30 @@ sub changed_hash {
     return $self->{changed_hash};
 }
 
-=item B<added>
-
-Returns the list of files added in the revision/transaction.
-
-=cut
 
 sub added {
     my $self = shift;
     return @{$self->changed_hash()->{added}};
 }
 
-=item B<updated>
-
-Returns the list of files updated in the revision/transaction.
-
-=cut
 
 sub updated {
     my $self = shift;
     return @{$self->changed_hash()->{updated}};
 }
 
-=item B<deleted>
-
-Returns the list of files deleted in the revision/transaction.
-
-=cut
 
 sub deleted {
     my $self = shift;
     return @{$self->changed_hash()->{deleted}};
 }
 
-=item B<prop_modified>
-
-Returns the list of files that had properties modified in the
-revision/transaction.
-
-=cut
 
 sub prop_modified {
     my $self = shift;
     return @{$self->changed_hash()->{prop_modified}};
 }
 
-=item B<changed>
-
-Returns the list of all files added, updated, deleted, and the ones
-that had properties modified in the revision/transaction.
-
-=cut
 
 sub changed {
     my $self = shift;
@@ -352,11 +166,33 @@ sub changed {
     return @{$hash->{changed}};
 }
 
-=item B<dirs_changed>
 
-Returns the list of directories changed in the revision/transaction.
+sub copied_to {
+    my $self = shift;
+    return keys %{$self->changed_hash()->{copied}};
+}
+
+
+sub copied_from {
+    my $self = shift;
+    return map {$_->[0]} values %{$self->changed_hash()->{copied}};
+}
+
+
+sub date {
+    my $self = shift;
+    unless (exists $self->{date}) {
+        $self->{date} = $self->_svnlook('date');
+    }
+    return $self->{date};
+}
+
+
+sub diff {
+    my ($self, @opts) = @_;
+    return $self->_svnlook('diff', @opts);
+}
 
-=cut
 
 sub dirs_changed {
     my $self = shift;
@@ -367,45 +203,233 @@ sub dirs_changed {
     return @{$self->{dirs_changed}};
 }
 
-=item B<copied_to>
 
-Returns the list of new names of files that were copied in the
-revision/transaction.
-
-=cut
-
-sub copied_to {
-    my $self = shift;
-    return keys %{$self->changed_hash()->{copied}};
+sub filesize {
+    my ($self, $path) = @_;
+    return $self->_svnlook('filesize', $path);
 }
 
-=item B<copied_from>
 
-Returns the list of original names of files that were copied in the
-revision/transaction. The order of this list is guaranteed to agree
-with the order generated by the method copied_to.
-
-=cut
-
-sub copied_from {
+sub info {
     my $self = shift;
-    return map {$_->[0]} values %{$self->changed_hash()->{copied}};
+    return $self->_svnlook('info');
 }
 
-=item B<cat> PATH
+
+sub lock {
+    my ($self, $path) = @_;
+    my %lock = ();
+    my @lock = $self->_svnlook('lock', $path);
+
+    while (my $line = shift @lock) {
+	chomp $line;
+	my ($key, $value) = split /:\s*/, $line, 2;
+	if ($key =~ /^Comment/) {
+	    $lock{Comment} = join('', @lock);
+	    last;
+	}
+	else {
+	    $lock{$key} = $value;
+	}
+    }
+
+    return %lock ? \%lock : undef;
+}
+
+
+sub log_msg {
+    my $self = shift;
+    unless (exists $self->{log}) {
+        $self->{log} = $self->_svnlook('log');
+    }
+    return $self->{log};
+}
+
+
+sub propget {
+    my ($self, @args) = @_;
+    return $self->_svnlook('propget', @args);
+}
+
+
+sub proplist {
+    my ($self, $path) = @_;
+    unless ($self->{proplist}{$path}) {
+        my $text = $self->_svnlook('proplist', '--verbose', $path);
+        my @list = split /^\s\s(\S+)\s:\s/m, $text;
+        shift @list;            # skip the leading empty field
+        chomp(my %hash = @list);
+        $self->{proplist}{$path} = \%hash;
+    }
+    return $self->{proplist}{$path};
+}
+
+
+sub tree {
+    my ($self, @args) = @_;
+    return $self->_svnlook('tree', @args);
+}
+
+
+sub uuid {
+    my ($self) = @_;
+    return $self->_svnlook('uuid');
+}
+
+
+sub youngest {
+    my ($self) = @_;
+    return $self->_svnlook('youngest');
+}
+
+1; # End of SVN::Look
+
+__END__
+=pod
+
+=head1 NAME
+
+SVN::Look - A caching wrapper around the svnlook command.
+
+=head1 VERSION
+
+version 0.34
+
+=head1 SYNOPSIS
+
+  use SVN::Look;
+  my $revlook = SVN::Look->new('/repo/path', -r => 123);
+  my $author  = $revlook->author();
+  my $msg     = $revlook->log_msg();
+  my @added_files   = $revlook->added();
+  my @updated_files = $revlook->updated();
+  my @deleted_files = $revlook->deleted();
+  my @changed_files = $revlook->changed();
+  my $file_contents = $revlook->cat('/path/to/file/in/repository');
+
+  my $trxlook = SVN::Look->new('/repo/path', -t => 123);
+
+=head1 DESCRIPTION
+
+The svnlook command is the workhorse of Subversion hook scripts, being
+used to gather all sorts of information about a repository, its
+revisions, and its transactions. This script provides a simple object
+oriented interface to a specific svnlook invocation, to make it easier
+to hook writers to get and use the information they need. Moreover,
+all the information gathered buy calling the svnlook command is cached
+in the object, avoiding repetitious calls.
+
+=head1 METHODS
+
+=head2 B<new> REPO [, WHAT, NUMBER]
+
+The SVN::Look constructor needs one or three arguments:
+
+=over
+
+=item REPO is the path to the repository.
+
+=item WHAT must be either '-r' or '-t', specifying if the third
+argument is a revision number or a transaction number, respectively.
+If neither -r or -t is specified, the HEAD revision is used.
+
+=item NUMBER is either a revision or transaction NUMBER, as specified
+by WHAT.
+
+=back
+
+=head2 B<repo>
+
+Returns the repository path that was passed to the constructor.
+
+=head2 B<txn>
+
+Returns the transaction number that was passed to the constructor. If
+none was passed, returns undef.
+
+=head2 B<rev>
+
+Returns the revision number that was passed to the constructor. If
+none was passed, returns undef.
+
+=head2 B<author>
+
+Returns the author of the revision/transaction.
+
+=head2 B<cat> PATH
 
 Returns the contents of the file at PATH. In scalar context, return
 the whole contents in a single string. In list context returns a list
 of chomped lines.
 
-=cut
+=head2 B<changed_hash>
 
-sub cat {
-    my ($self, $path) = @_;
-    return $self->_svnlook('cat', $path);
-}
+Returns a reference to a hash containing information about all file
+changes occurred in the revision. The hash always has the following
+keys:
 
-=item B<diff> [OPTS, ...]
+=over
+
+=item added
+
+A list of files added in the revision.
+
+=item deleted
+
+A list of files deleted in the revision.
+
+=item updated
+
+A list of files updated in the revision.
+
+=item prop_modified
+
+A list of files that had properties modified in the revision.
+
+=item copied
+
+A hash containing information about each file or diretory copied in the revision. The hash keys are the names of elements copied to. The value associated with a key is a two-element array containing the name of the element copied from and the specific revision from which it was copied.
+
+=back
+
+=head2 B<added>
+
+Returns the list of files added in the revision/transaction.
+
+=head2 B<updated>
+
+Returns the list of files updated in the revision/transaction.
+
+=head2 B<deleted>
+
+Returns the list of files deleted in the revision/transaction.
+
+=head2 B<prop_modified>
+
+Returns the list of files that had properties modified in the
+revision/transaction.
+
+=head2 B<changed>
+
+Returns the list of all files added, updated, deleted, and the ones
+that had properties modified in the revision/transaction.
+
+=head2 B<copied_to>
+
+Returns the list of new names of files that were copied in the
+revision/transaction.
+
+=head2 B<copied_from>
+
+Returns the list of original names of files that were copied in the
+revision/transaction. The order of this list is guaranteed to agree
+with the order generated by the method copied_to.
+
+=head2 B<date>
+
+Returns the date of the revision/transaction.
+
+=head2 B<diff> [OPTS, ...]
 
 Returns the GNU-style diffs of changed files and properties. There are
 three optional options that can be passed as strings:
@@ -429,36 +453,21 @@ Print differences against the copy source.
 In scalar context, return the whole diff in a single string. In list
 context returns a list of chomped lines.
 
-=cut
+=head2 B<dirs_changed>
 
-sub diff {
-    my ($self, @opts) = @_;
-    return $self->_svnlook('diff', @opts);
-}
+Returns the list of directories changed in the revision/transaction.
 
-=item B<youngest>
+=head2 B<filesize> PATH
 
-Returns the repository's youngest revision number.
+Returns the size (in bytes) of the file located at PATH as it is
+represented in the repository.
 
-=cut
+=head2 B<info>
 
-sub youngest {
-    my ($self) = @_;
-    return $self->_svnlook('youngest');
-}
+Returns the author, datestamp, log message size, and log message of
+the revision/transaction.
 
-=item B<uuid>
-
-Returns the repository's UUID.
-
-=cut
-
-sub uuid {
-    my ($self) = @_;
-    return $self->_svnlook('uuid');
-}
-
-=item B<lock> PATH
+=head2 B<lock> PATH
 
 If PATH has a lock, returns a hash containing information about the lock, with the following keys:
 
@@ -484,29 +493,19 @@ The lock comment.
 
 If PATH has no lock, returns undef.
 
-=cut
+=head2 B<log_msg>
 
-sub lock {
-    my ($self, $path) = @_;
-    my %lock = ();
-    my @lock = $self->_svnlook('lock', $path);
+Returns the log message of the revision/transaction.
 
-    while (my $line = shift @lock) {
-	chomp $line;
-	my ($key, $value) = split /:\s*/, $line, 2;
-	if ($key =~ /^Comment/) {
-	    $lock{Comment} = join('', @lock);
-	    last;
-	}
-	else {
-	    $lock{$key} = $value;
-	}
-    }
+=head2 B<propget> PROPNAME PATH
 
-    return %lock ? \%lock : undef;
-}
+Returns the value of PROPNAME in PATH.
 
-=item B<tree> [PATH_IN_REPOS, OPTS, ...]
+=head2 B<proplist> PATH
+
+Returns a reference to a hash containing the properties associated with PATH.
+
+=head2 B<tree> [PATH_IN_REPOS, OPTS, ...]
 
 Returns the repository tree as a list of paths, starting at
 PATH_IN_REPOS (if supplied, at the root of the tree otherwise),
@@ -528,60 +527,24 @@ Operate on single directory only.
 
 =back
 
-=cut
+=head2 B<uuid>
 
-sub tree {
-    my ($self, @opts) = @_;
-    return $self->_svnlook('tree', @opts);
-}
+Returns the repository's UUID.
 
-=back
+=head2 B<youngest>
+
+Returns the repository's youngest revision number.
 
 =head1 AUTHOR
 
-Gustavo Chaves, C<< <gnustavo@cpan.org> >>
+Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
-=head1 BUGS
+=head1 COPYRIGHT AND LICENSE
 
-Please report any bugs or feature requests to C<bug-svn-look at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=SVN-Hooks>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+This software is copyright (c) 2012 by CPqD.
 
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc SVN::Look
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=SVN-Hooks>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/SVN-Hooks>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/SVN-Hooks>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/SVN-Hooks>
-
-=back
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008-2011 CPqD, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-1; # End of SVN::Look
